# -*- coding: utf-8 -*-
"""The game modue!
    Here is the location of the infamous Main Loop!"""
    
    
import pygame
from pygame.constants import *

import gui
import utils
import data
import areas
import env
import sound
import parser

class Game:
    def __init__(self,  screen):
        self.screen = screen
        self.screenrect = screen.get_rect()
        self.quit = False
        self.clock = pygame.time.Clock()

        self.input = gui.InputBox(parser.Parser(parser.command_aliases), (20, 550), (360, 30))
        self.input.rect.centerx = self.screenrect.centerx

        self.messages = gui.MessageBox((0, 0), (600, 100))
        self.messages.rect.centerx = self.screenrect.centerx
        self.messages.rect.bottom = self.input.rect.top - 20

        self.gui = pygame.sprite.Group()
        self.gui.add(self.input)
        self.gui.add(self.messages)

        self.typing = (False, None, None)
        
        self.background = lambda: None

    def _begin_typing(self, fun):
        """(Typing?, function to call, time when typing began)"""
        t = pygame.time.get_ticks()
        fun() #Run the funtion once
        self.typing = (True, fun, t)

    def check_input(self):
        for event in pygame.event.get():
            if event.type == QUIT:
                self.quit = True

            if event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    self.quit = True

                if event.mod & KMOD_LCTRL or event.mod & KMOD_RCTRL:
                    if event.key == K_m:
                        sound.toggle_mute()

                    # This is supposed to be fullscreen, and it seems to work for most people
                    # I get an OUT OF RANGE error message from the screen. If 
                    # anyone has any suggestion on how to fix this, please feel free
                    # to mail it to me.

                    if event.key == K_f:
                        pygame.display.toggle_fullscreen()

                # Also fullscreen
                elif event.mod & KMOD_LALT or event.mod & KMOD_LALT:
                    if event.key == K_RETURN:
                        pygame.display.toggle_fullscreen()
                
                else: # Keys without modifiers mentioned above
                    if event.key == K_PAGEUP:
                        self._begin_typing(lambda: self.messages.move_history(-1))
                    if event.key == K_PAGEDOWN:
                        self._begin_typing(lambda: self.messages.move_history(1))

                    if event.key == K_UP:
                        self._begin_typing(lambda: self.input.move_history(-1))
                    if event.key == K_DOWN:
                        self._begin_typing(lambda: self.input.move_history(1))

                    if event.key == K_LEFT:
                        self._begin_typing(lambda: self.input.move(-1))
                    if event.key == K_RIGHT:
                        self._begin_typing(lambda: self.input.move(1))

                    if event.key == K_RETURN:
                        self.input.confirm()
                    if event.key == K_BACKSPACE:
                        self._begin_typing(lambda: self.input.delete_back())
                    if event.key == K_DELETE:
                        self._begin_typing(lambda: self.input.delete_front())

                    if gui.typable(event.key):
                        ch = chr(ord(u"a") + event.key - K_a)
                        if event.mod & KMOD_LSHIFT or event.mod & KMOD_RSHIFT or event.mod & KMOD_CAPS:
                            if ch == "-" : ch = "_"
                            else: ch = ch.upper()
                        self._begin_typing(lambda: self.input.addch(ch))

            if event.type == KEYUP:
                self.typing = (False, None, None)

            # This is where messages that we want to output end up
            # We added a custom event to the SDL event queue to 
            # overcome the problem of implementing a simple way
            # to return messages and descriptions from all parts
            # of the game whenever they want to say something 
            # relevant. An event is logical, because a message
            # is equivalent to something yelling for attention.
            # I'm da bro in da hood, savin' yo code.
            if event.type == USEREVENT:
                if event.message_type == "output":
                    self.messages.add(event.message)

        # For keys that can be held down
        if self.typing[0]:
            dt = pygame.time.get_ticks() - self.typing[2]
            if dt > 400:         # If the time threshold has been passed 
                self.typing[1]() # we repeat the command

    def run(self):

        while not self.quit:
            self.check_input()
   
            area = env.env["current_area"]
            self.screen.blit(area.image(),  (0, 0))

            self.input.update()
            self.gui.draw(self.screen)

            pygame.display.flip()
            self.clock.tick(24)
            

class NormalGame(Game):
    def __init__(self, screen):
        Game.__init__(self,  screen)

        env.env["current_area"] = areas.areas["your_cottage"]
        
    def run(self):
        utils.add_message(env.env["current_area"].enter())
        Game.run(self)
