# -*- coding: utf-8 -*-
'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "gamelib"
package.
'''
import pygame
from pygame.locals import *
from optparse import OptionParser
import sys

import data
import sound
import utils
import introutro

_prog_name_ = "Heartfelt nightsong"
_version_ = "pyweek final version"

def init_pygame(flags):
    pygame.init()
    
    screen = pygame.display.set_mode((800, 600),  flags | DOUBLEBUF)
    pygame.display.set_caption(_prog_name_)
    pygame.mouse.set_visible(False)

    import atexit
    atexit.register(pygame.quit)

    return screen

def main():
    
    parser = OptionParser(usage="Usage: %prog [options] [savefile]", version=_prog_name_ + " " + _version_)
    parser.add_option("-f", "--fullscreen", dest="fullscreen",
                      action="store_true", default=False,
                      help="start game in fullscreen")
    parser.add_option("-m", "--mute", dest="mute",
                      action="store_true", default=False,
                      help="turn off sound")
    parser.add_option("-l", "--load", dest="savefile",
                      help="load save file", metavar="FILE")

    (options, args) = parser.parse_args()

    flags = 0
    if options.fullscreen:
        flags |= FULLSCREEN

    sound.ismuting = options.mute

    screen = init_pygame(flags)

    savefile = None
    if options.savefile:
        savefile = options.savefile
    elif len(args) > 0:
        savefile = args[0]

    import game
    g = game.NormalGame(screen)
    
    # Load game after game has been initialized
    if savefile is not None:
        errcode, err = utils.load_game(savefile)
        if errcode is not 0:
            print "Could not load file:", err
            sys.exit(1)
    else: 
        introutro.play_intro()
        
    g.run()
